@@ -1,12 +1,37 @@
 #! /bin/sh
 
 top=`pwd`
-archive=${1-gcc-4.7.0.tar.bz2}
+gcc_version=4.7.1
 contact="Andris Pavenis <andris.pavenis@iki.fi>";
 
-#export SHELL=$DJDIR/bin/sh.exe
+# Configuration params
+create_gccnnns_zip=true
 
-#exec >unpack-`date +%y%m%d` 2>&1
+params_done=false
+while ! $params_done ; do
+    case "x$1" in
+        x--*)
+            case $1 in
+                --no-djgpp-source)
+                    create_gccnnns_zip=false
+                    ;;
+                --verbose-shell)
+                    set -x
+                    ;;
+                *)
+                    echo "Unrecognized command line option $1"
+                    exit 1
+                    ;;
+            esac
+            shift
+            ;;
+    *)
+        params_done=true
+        ;;
+    esac
+done
+
+archive=${1-gcc-$gcc_version.tar.bz2}
 
 rm -f $top/s-sources
 rm -f $top/s-patch
@@ -21,7 +46,7 @@ case $archive in
          ( cd gnu && tar xf $top/$archive --use=bzip2 && echo $archive >$top/s-sources )
          ;;
       *)
-         echo Unknown archive format
+         echo "Unknown archive format ($top/$archive)"
 	 exit 1	
          ;; 
 esac
@@ -110,17 +135,80 @@ aclocal_list=
 #  Apply DJGPP related diffs and copy files 
 #
 
+#
+#  File copy instructions required for building cross-compiler
+#
+file_copy_list_1=
+
+#
+#  Additional file copy instructions required for building native-compiler
+#
+
+if $create_gccnnns_zip ; then
+    file_copy_list_2="gcc/ada/system.ads:system-djgpp.ads";
+else
+    file_copy_list_2=;
+fi
+
 patching_ok=true
-for x in `(cd diffs/source && find . -type f | sed -e 's/\.\///g')`; do
+
+for copy_instr in $file_copy_list_1 $file_copy_list_2 ; do
+    orig_fn=$(echo $copy_instr | sed -e 's/:.*$//')
+    orig_dir=$(dirname $orig_fn)
+    new_base_name=$(basename $(echo $copy_instr | sed -e 's/^[^:]*://'))
+    new_fn=$orig_dir/$new_base_name
+    if [ -f $new_fn ] ; then
+        echo "ERROR: File $new_fn already exists"
+        patching_ok=false
+    fi
+    if ! [ -f $gcc_dir/$orig_fn ] ; then
+        echo "ERROR: File $orig_fn is not found"
+        patching_ok=false
+    fi
+    cp -Pv $gcc_dir/$orig_fn $gcc_dir/$new_fn
+done 
+    
+if ! $patching_ok ; then
+    exit 1
+fi
+
+#
+#  Patch list required for building cross-compiler
+#
+patch_instr_list_1=$(find diffs/source -type f | sed -e 's/\.\///g')
+
+#
+#  Additional patch list required for building native compiler for DJGPP
+#
+if $create_gccnnns_zip; then
+    patch_instr_list_2=$(find diffs2/source -type f | sed -e 's/\.\///g');
+else
+    patch_instr_list_2=;
+fi
+   
+for x in $patch_instr_list_1 $patch_instr_list_2; do
+    patch_src=$x
+    case $x in
+        diffs/*)
+            backup_name=.orig0
+            ;;
+        diffs2/*)
+            backup_name=.orig1
+            ;;
+    esac
+
+    x=$(echo $x | sed -e 's:diffs2*/source/::')
+
     case $x in 
        *.diff | *.diff.in)
            cd $top
            diff=`pwd`/$x;
            fname=`echo $x | sed -e 's/\.diff//g'`
+
            echo "Applying patch to gcc/$fname ..."
            if [ -f $gcc_dir/$fname ] ; then
               cp -f $gcc_dir/$fname $gcc_dir/${fname}~1 
-              if ! patch --verbose $gcc_dir/$fname <diffs/source/$x ; then
+              if ! patch --verbose --suffix=$backup_name $gcc_dir/$fname <$patch_src ; then
                   echo "Failed to apply patch for $gcc_dir/$fname"
 		  patching_ok=false
               fi
@@ -150,9 +238,13 @@ for x in `(cd diffs/source && find . -type f | sed -e 's/\.\///g')`; do
            ;;
        *)
            echo Copying $x
-	   mkdir -p `dirname $gcc_dir/$x`		   	
-           if ! cp -fv diffs/source/$x $gcc_dir/$x ; then
-              echo "Failed to copy diffs/source/$x to $gcc_dir/$x"
+           if [ -f $gcc_dir/$x ] ; then
+               echo "ERROR: File gcc_dir/$x already exists"
+               exit 1
+           fi
+	   mkdir -p $(dirname $gcc_dir/$x)		   	
+           if ! cp -fv $patch_src $gcc_dir/$x ; then
+              echo "Failed to copy $patch_src to $gcc_dir/$x"
    	      patching_ok=false
            fi
            ;;
@@ -161,14 +253,16 @@ done
 
 $patching_ok || exit 1
 
-for dir in build.gcc install.gcc; do
-    for x in `(cd diffs/$dir && find . -type f | sed -e 's/\.\///g')`; do
-        mkdir -p `dirname gnu/$dir/$x`
-	echo "Writting $dir/$x"
-	cat diffs/$dir/$x |\
-	sed -e "s:@SRCDIR@:$dir_name:g" -e "s:@GCCVER@:$short_version:g" >gnu/$dir/$x
+if $create_gccnnns_zip; then
+    for dir in build.gcc install.gcc; do
+        for x in `(cd diffs2/$dir && find . -type f | sed -e 's/\.\///g')`; do
+            mkdir -p `dirname gnu/$dir/$x`
+	    echo "Writting $dir/$x"
+	    cat diffs2/$dir/$x |\
+	    sed -e "s:@SRCDIR@:$dir_name:g" -e "s:@GCCVER@:$short_version:g" >gnu/$dir/$x
+        done
     done
-done
+fi
 
 #
 #  Run aclocal where needed
@@ -255,17 +349,19 @@ done
 
 cd $top
 
-mkdir manifest
-echo ${arcprefix}s${version_status}.zip - GCC $version source archive for DJGPP >$top/manifest/${arcprefix}s${version_status}.ver
-touch $top/manifest/${arcprefix}s${version_status}.mft
+if $create_gccnnns_zip; then
+    mkdir manifest
+    echo ${arcprefix}s${version_status}.zip - GCC $version source archive for DJGPP >$top/manifest/${arcprefix}s${version_status}.ver
+    touch $top/manifest/${arcprefix}s${version_status}.mft
 
-( find gnu -type f \
-          -and -not -name '.cvsignore' \
-          -and -not -name '*~*' \
-          -and -not -name '*.orig' \
-          -and -not -name '.brik' ;
-  find manifest -type f; ) |\
-  grep -v 'autom4te\.cache' |\
-  sort >$top/manifest/${arcprefix}s${version_status}.mft
+    (
+        find gnu -type f \
+            -and -not -name '.cvsignore' \
+            -and -not -name '*~*' \
+            -and -not -name '*.orig' \
+            -and -not -name '.brik' ;
+        find manifest -type f;
+    ) | grep -v 'autom4te\.cache' | sort >$top/manifest/${arcprefix}s${version_status}.mft
 
-cat $top/manifest/${arcprefix}s${version_status}.mft | zip -9q@ $top/${arcprefix}s${version_status}.zip
+    cat $top/manifest/${arcprefix}s${version_status}.mft | zip -9q@ $top/${arcprefix}s${version_status}.zip
+fi
